﻿namespace TabsStudioSQLComment
{
    class TabManager
    {
        public TabManager(Options options_)
        {
            options = options_;
            comments = new System.Collections.Generic.Dictionary<EnvDTE.Window, string>();
        }

        public void OnTabCreated(TabsStudioExt.TabEventArgs e)
        {
            AddDocumentComment(e.Tab.TitleManager, e.Tab.TabExtensions[0].Window);
        }

        public void OnTabExtensionCreated(TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabExtensions.Count == 2)
                RemoveComment(e.Tab.TitleManager);
            AddDocumentComment(e.TabExtension.TitleManager, e.TabExtension.Window);
        }

        public void OnTabExtensionDestroyed(TabsStudioExt.TabExtensionEventArgs e)
        {
            comments.Remove(e.TabExtension.Window);
            if (e.Tab.TabExtensions.Count == 2)
            {
                TabsStudioExt.TabExtension remainingExtension = e.Tab.TabExtensions[0].Window == e.TabExtension.Window ?
                        e.Tab.TabExtensions[1] : e.Tab.TabExtensions[0];
                UpdateComment(e.Tab.TitleManager, remainingExtension.Window);
            }
        }

        public void SetTabComment(TabsStudioExt.Tab tab, string comment)
        {
            TabsStudioExt.TabExtension ext = tab.TabExtensions[0];
            comments[ext.Window] = comment;
            UpdateComment(tab.TitleManager, ext.Window);
            UpdateComment(ext.TitleManager, ext.Window);
        }

        public void SetTabExtensionComment(TabsStudioExt.TabExtension ext, string comment)
        {
            comments[ext.Window] = comment;
            UpdateComment(ext.TitleManager, ext.Window);
        }

        public string GetComment(EnvDTE.Window w)
        {
            if (!comments.ContainsKey(w))
                return null;

            return comments[w];
        }

        private void UpdateComment(TabsStudioExt.ITitleManager titleManager, EnvDTE.Window w)
        {
            RemoveComment(titleManager);
            if (comments.ContainsKey(w))
                AddComment(titleManager, comments[w]);
        }

        private void AddDocumentComment(TabsStudioExt.ITitleManager titleManager, EnvDTE.Window w)
        {
            if (!IsPoorlyNamedTab(titleManager))
                return;

            string comment = GetCommentFromDocument(w);
            if (comment != null)
            {
                comments[w] = comment;
                UpdateComment(titleManager, w);
            }
        }

        private string GetCommentFromDocument(EnvDTE.Window w)
        {
            try
            {
                System.Text.RegularExpressions.Regex regex =
                    new System.Text.RegularExpressions.Regex(options.commentRegex,
                        System.Text.RegularExpressions.RegexOptions.Multiline);
                System.Text.RegularExpressions.Match match =
                    regex.Match(GetDocumentText(w.Document));

                string comment = "";
                foreach (System.Text.RegularExpressions.Capture c in match.Groups["M"].Captures)
                    comment += c.Value;
                comment = comment.Replace("\r", "").Replace("\n", "");

                if (comment.Length > 0)
                    return comment;
            }
            catch (System.Exception)
            {
            }
            return null;
        }

        private bool IsPoorlyNamedTab(TabsStudioExt.ITitleManager titleManager)
        {
            System.Text.RegularExpressions.Regex regex =
                new System.Text.RegularExpressions.Regex(options.tabNameFilter);
            return regex.IsMatch(titleManager.UntransformedTitle);
        }

        // throws System.Exception
        private string GetDocumentText(EnvDTE.Document doc)
        {
            EnvDTE.TextDocument textDoc = doc.Object("TextDocument") as EnvDTE.TextDocument;
            if (textDoc == null)
                return "";
            return textDoc.StartPoint.CreateEditPoint().GetText(textDoc.EndPoint);
        }

        private void AddComment(TabsStudioExt.ITitleManager titleManager, string comment)
        {
            titleManager.Transforms.Add(
                new TabsStudioExt.TitleTransform(options.transformPattern,
                    options.transformReplacement.Replace("${SQLComment}", comment),
                    transformTag));
            titleManager.ApplyTransforms();
        }

        private void RemoveComment(TabsStudioExt.ITitleManager titleManager)
        {
            foreach (TabsStudioExt.TitleTransform t in titleManager.Transforms)
            {
                if (t.Id == transformTag)
                {
                    titleManager.Transforms.Remove(t);
                    titleManager.ApplyTransforms();
                    return;
                }
            }
        }

        private Options options;
        private System.Collections.Generic.Dictionary<EnvDTE.Window, string> comments; 
        private static string transformTag = "[SQLComment]";
    }
}
